@@ -16,6 +16,7 @@ from uuid import uuid4
 import requests
 import shortuuid
 import yaml
+from fastapi.exceptions import HTTPException
 from github import GithubException
 from github.Branch import Branch
 from github.CheckRun import CheckRun
@@ -27,6 +28,7 @@ from stringcolor import cs
 from timeout_sampler import TimeoutExpiredError, TimeoutSampler
 
 from webhook_server.libs.config import Config
+from webhook_server.libs.exceptions import NoPullRequestError, RepositoryNotFoundError
 from webhook_server.utils.constants import (
     ADD_STR,
     APPROVE_STR,
@@ -80,22 +82,6 @@ from webhook_server.utils.helpers import (
 )
 
 
-class NoPullRequestError(Exception):
-    pass
-
-
-class RepositoryNotFoundError(Exception):
-    pass
-
-
-class ProcessGithubWehookError(Exception):
-    def __init__(self, err: dict[str, str]):
-        self.err = err
-
-    def __str__(self) -> str:
-        return f"{self.err}"
-
-
 class ProcessGithubWehook:
     def __init__(self, hook_data: dict[Any, Any], headers: Headers, logger: logging.Logger) -> None:
         self.logger = logger
@@ -112,13 +98,24 @@ class ProcessGithubWehook:
         self.owners_content: dict[str, Any] = {}
 
         self.config = Config(repository=self.repository_name)
-        self._repo_data_from_config()
+
+        if not self.config.repository:
+            raise RepositoryNotFoundError(f"Repository {self.repository_name} not found in config file")
+
+        # Get config without .github-webhook-server.yaml data
+        self._repo_data_from_config(repository_config={})
         self.github_api, self.token, self.api_user = get_api_with_highest_rate_limit(
             config=self.config, repository_name=self.repository_name
         )
 
         if self.github_api and self.token:
             self.repository = get_github_repo_api(github_api=self.github_api, repository=self.repository_full_name)
+            # Once we have a repository, we can get the config from .github-webhook-server.yaml
+            local_repository_config = self.config.repository_local_data(
+                github_api=self.github_api, repository_full_name=self.repository_full_name
+            )
+            # Call _repo_data_from_config() again to update self args from .github-webhook-server.yaml
+            self._repo_data_from_config(repository_config=local_repository_config)
 
         else:
             self.logger.error(f"Failed to get GitHub API and token for repository {self.repository_name}.")
@@ -155,9 +152,9 @@ class ProcessGithubWehook:
             "Report bugs in [Issues](https://github.com/myakove/github-webhook-server/issues)"
         )
 
-    def process(self) -> None:
+    async def process(self) -> Any:
         if self.github_event == "ping":
-            return
+            return {"status": requests.codes.ok, "message": "pong"}
 
         event_log: str = f"Event type: {self.github_event}. event ID: {self.x_github_delivery}"
 
@@ -175,22 +172,28 @@ class ProcessGithubWehook:
             self.all_reviewers = self.get_all_reviewers()
 
             if self.github_event == "issue_comment":
-                self.process_comment_webhook_data()
+                return self.process_comment_webhook_data()
 
-            elif self.github_event == "pull_request":
-                self.process_pull_request_webhook_data()
+            if self.github_event == "pull_request":
+                return self.process_pull_request_webhook_data()
 
-            elif self.github_event == "pull_request_review":
-                self.process_pull_request_review_webhook_data()
+            if self.github_event == "pull_request_review":
+                return self.process_pull_request_review_webhook_data()
 
-            elif self.github_event == "check_run":
-                self.process_pull_request_check_run_webhook_data()
+            if self.github_event == "check_run":
+                return self.process_pull_request_check_run_webhook_data()
 
         except NoPullRequestError:
             self.logger.debug(f"{self.log_prefix} {event_log}. [No pull request found in hook data]")
 
             if self.github_event == "push":
-                self.process_push_webhook_data()
+                return self.process_push_webhook_data()
+
+            raise
+
+        except Exception as e:
+            self.logger.error(f"{self.log_prefix} {event_log}. Exception: {e}")
+            raise HTTPException(status_code=404, detail=str(e))
 
     @property
     def _prepare_retest_welcome_comment(self) -> str:
@@ -270,6 +273,7 @@ class ProcessGithubWehook:
 
     def prepare_log_prefix(self, pull_request: PullRequest | None = None) -> str:
         _repository_color = self._get_reposiroty_color_for_log_prefix()
+
         return (
             f"{_repository_color}[{self.github_event}][{self.x_github_delivery}][{self.api_user}][PR {pull_request.number}]:"
             if pull_request
@@ -298,19 +302,21 @@ class ProcessGithubWehook:
 
         return self.check_if_can_be_merged()
 
-    def _repo_data_from_config(self) -> None:
-        if not self.config.repository:
-            raise RepositoryNotFoundError(f"Repository {self.repository_name} not found in config file")
+    def _repo_data_from_config(self, repository_config: dict[str, Any]) -> None:
+        self.logger.debug(f"Read config for repository {self.repository_name}")
 
-        self.github_app_id: str = self.config.get_value(value="github-app-id")
+        self.github_app_id: str = self.config.get_value(value="github-app-id", extra_dict=repository_config)
+        self.pypi: dict[str, str] = self.config.get_value(value="pypi", extra_dict=repository_config)
+        self.verified_job: bool = self.config.get_value(
+            value="verified-job", return_on_none=True, extra_dict=repository_config
+        )
+        self.tox: dict[str, str] = self.config.get_value(value="tox", extra_dict=repository_config)
+        self.tox_python_version: str = self.config.get_value(value="tox-python-version", extra_dict=repository_config)
+        self.slack_webhook_url: str = self.config.get_value(value="slack_webhook_url", extra_dict=repository_config)
 
-        self.pypi: dict[str, str] = self.config.get_value(value="pypi")
-        self.verified_job: bool = self.config.get_value(value="verified-job", return_on_none=True)
-        self.tox: dict[str, str] = self.config.get_value(value="tox")
-        self.tox_python_version: str = self.config.get_value(value="tox-python-version")
-        self.slack_webhook_url: str = self.config.get_value(value="slack_webhook_url")
-
-        self.build_and_push_container: dict[str, Any] = self.config.get_value(value="container", return_on_none={})
+        self.build_and_push_container: dict[str, Any] = self.config.get_value(
+            value="container", return_on_none={}, extra_dict=repository_config
+        )
         if self.build_and_push_container:
             self.container_repository_username: str = self.build_and_push_container["username"]
             self.container_repository_password: str = self.build_and_push_container["password"]
@@ -321,17 +327,23 @@ class ProcessGithubWehook:
             self.container_command_args: str = self.build_and_push_container.get("args", "")
             self.container_release: bool = self.build_and_push_container.get("release", False)
 
-        self.pre_commit: bool = self.config.get_value(value="pre-commit", return_on_none=False)
+        self.pre_commit: bool = self.config.get_value(
+            value="pre-commit", return_on_none=False, extra_dict=repository_config
+        )
 
         self.auto_verified_and_merged_users: list[str] = self.config.get_value(
-            value="auto-verified-and-merged-users", return_on_none=[]
+            value="auto-verified-and-merged-users", return_on_none=[], extra_dict=repository_config
         )
         self.can_be_merged_required_labels = self.config.get_value(
-            value="can-be-merged-required-labels", return_on_none=[]
+            value="can-be-merged-required-labels", return_on_none=[], extra_dict=repository_config
+        )
+        self.conventional_title: str = self.config.get_value(value="conventional-title", extra_dict=repository_config)
+        self.set_auto_merge_prs: list[str] = self.config.get_value(
+            value="set-auto-merge-prs", return_on_none=[], extra_dict=repository_config
+        )
+        self.minimum_lgtm: int = self.config.get_value(
+            value="minimum-lgtm", return_on_none=0, extra_dict=repository_config
         )
-        self.conventional_title: str = self.config.get_value(value="conventional-title")
-        self.set_auto_merge_prs: list[str] = self.config.get_value(value="set-auto-merge-prs", return_on_none=[])
-        self.minimum_lgtm: int = self.config.get_value(value="minimum-lgtm", return_on_none=0)
 
     def _get_pull_request(self, number: int | None = None) -> PullRequest:
         if number:
@@ -845,9 +857,10 @@ Publish to PYPI failed: `{_error}`
             _check_for_merge: bool = False
             _user: str | None = None
             action_labeled = hook_action == "labeled"
-            labeled = self.hook_data["label"]["name"].lower()
+            labeled = self.hook_data["label"]["name"]
+            labeled_lower = labeled.lower()
 
-            if labeled == CAN_BE_MERGED_STR:
+            if labeled_lower == CAN_BE_MERGED_STR:
                 return
 
             self.logger.info(f"{self.log_prefix} PR {self.pull_request.number} {hook_action} with {labeled}")
@@ -865,7 +878,7 @@ Publish to PYPI failed: `{_error}`
                     if _user in self.all_reviewers + self.all_approvers:
                         _check_for_merge = True
 
-            if self.verified_job and labeled == VERIFIED_LABEL_STR:
+            if self.verified_job and labeled_lower == VERIFIED_LABEL_STR:
                 _check_for_merge = True
 
                 if action_labeled:
@@ -957,8 +970,7 @@ Publish to PYPI failed: `{_error}`
             return
 
         if self.is_check_run_in_progress(check_run=TOX_STR):
-            self.logger.debug(f"{self.log_prefix} Check run is in progress, not running {TOX_STR}.")
-            return
+            self.logger.debug(f"{self.log_prefix} Check run is in progress, re-running {TOX_STR}.")
 
         clone_repo_dir = f"{self.clone_repo_dir}-{uuid4()}"
         python_ver = f"--python={self.tox_python_version}" if self.tox_python_version else ""
@@ -993,8 +1005,7 @@ Publish to PYPI failed: `{_error}`
             return
 
         if self.is_check_run_in_progress(check_run=PRE_COMMIT_STR):
-            self.logger.debug(f"{self.log_prefix} Check run is in progress, not running {PRE_COMMIT_STR}.")
-            return
+            self.logger.debug(f"{self.log_prefix} Check run is in progress, re-running {PRE_COMMIT_STR}.")
 
         clone_repo_dir = f"{self.clone_repo_dir}-{uuid4()}"
         cmd = f" uvx --directory {clone_repo_dir} {PRE_COMMIT_STR} run --all-files"
@@ -1331,13 +1342,15 @@ Publish to PYPI failed: `{_error}`
         if not self.build_and_push_container:
             return
 
+        if self.is_check_run_in_progress(check_run=BUILD_CONTAINER_STR):
+            self.logger.info(f"{self.log_prefix} Check run is in progress, re-running {BUILD_CONTAINER_STR}.")
+
         clone_repo_dir = f"{self.clone_repo_dir}-{uuid4()}"
         pull_request = hasattr(self, "pull_request")
 
         if pull_request and set_check:
             if self.is_check_run_in_progress(check_run=BUILD_CONTAINER_STR) and not is_merged:
-                self.logger.info(f"{self.log_prefix} Check run is in progress, not running {BUILD_CONTAINER_STR}.")
-                return
+                self.logger.info(f"{self.log_prefix} Check run is in progress, re-running {BUILD_CONTAINER_STR}.")
 
             self.set_container_build_in_progress()
 
@@ -1418,8 +1431,7 @@ Publish to PYPI failed: `{_error}`
             return
 
         if self.is_check_run_in_progress(check_run=PYTHON_MODULE_INSTALL_STR):
-            self.logger.info(f"{self.log_prefix} Check run is in progress, not running {PYTHON_MODULE_INSTALL_STR}.")
-            return
+            self.logger.info(f"{self.log_prefix} Check run is in progress, re-running {PYTHON_MODULE_INSTALL_STR}.")
 
         clone_repo_dir = f"{self.clone_repo_dir}-{uuid4()}"
         self.logger.info(f"{self.log_prefix} Installing python module")
@@ -2172,6 +2184,10 @@ Adding label/s `{" ".join([_cp_label for _cp_label in cp_labels])}` for automati
             "summary": "",
             "text": "",
         }
+
+        if self.is_check_run_in_progress(check_run=CONVENTIONAL_TITLE_STR):
+            self.logger.info(f"{self.log_prefix} Check run is in progress, re-running {CONVENTIONAL_TITLE_STR}.")
+
         self.set_conventional_title_in_progress()
         allowed_names = self.conventional_title.split(",")
         title = self.pull_request.title
